package Generate::Attlist;

# Module:  Generate::Attlist
# Purpose: Generates an ATTLIST DTD entry
# Author:  John Storrs, Laboratory for Micro Enterprise
# License: FSF GPL
# Version: $Id: Attlist.pm,v 1.2 2005/07/10 11:37:07 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   our @EXPORT_OK = qw(generateAttlist);
}

use LME::XmlParser;
use LME::XmlNode;
use Generate::Main qw(%hasAttributes);

# prototypes
sub generateAttlist($);


sub generateAttlist ($)
{
   my ($element) = @_;
   my ($result, $hasAttributes, $attributeContent, 
       $attribute, $attributeDefinition, $contentCount, $orderNumber, 
       $orderStep,
       @attributeContent, @attributes, @contentCounts,
       @allAttributes, @orderedAttributes,
       %isAttribute, %isRequired, %contentByCount, %orderNumber);

   # build sorted list of all element attributes
   %isAttribute = ();
   @attributeContent = keys(%hasAttributes);
   foreach $attributeContent (@attributeContent) {
      @attributes = split(', ', $attributeContent);
      foreach $attribute (@attributes) {
         $isAttribute{$attribute} = 1;
      }
   }
   @allAttributes = sort(keys(%isAttribute));

   # find if attribute is required or optional
   %isRequired = ();
   foreach $attribute (@allAttributes) {
      $isRequired{$attribute} = 1;  # initial value
   }
   foreach $attributeContent (@attributeContent) {
      @attributes = split(', ', $attributeContent);
      foreach $attribute (@allAttributes) {
         if (!grep(/$attribute/, @attributes)) {
            $isRequired{$attribute} = 0;
         }
      }
   }

   # organise attribute content by decreasing content count
   %contentByCount = ();
   foreach $attributeContent (@attributeContent) {
      $contentCount = $hasAttributes{$attributeContent};
      push(@{$contentByCount{$contentCount}}, $attributeContent);
   }
   @contentCounts = reverse sort numerically keys(%contentByCount);
   
   # from most comment content down, allocate order numbers
   $orderNumber = 0;
   $orderStep = 1000000;
   %orderNumber = ();
   #print("element tag = $element:\n");
   foreach $contentCount (@contentCounts) {
      foreach $attributeContent (@{$contentByCount{$contentCount}}) {
         #print("   attribute content = $attributeContent ($contentCount)\n");
         foreach $attribute (split(', ', $attributeContent)) {
            if (defined($orderNumber{$attribute})) {
               $orderNumber = $orderNumber{$attribute} + $orderStep;
            } else {
               $orderNumber{$attribute} = $orderNumber;
               $orderNumber += $orderStep;
            }
            #print("      attribute = $attribute, ");
            #print("order# = $orderNumber{$attribute}\n");
         }
         $orderStep /= 100;
      }
   }

   # invert %orderNumber to get ordered attribute list
   @orderedAttributes = ();
   foreach $orderNumber (sort numerically values(%orderNumber)) {
      foreach $attribute (@allAttributes) {
         if ($orderNumber{$attribute} == $orderNumber) {
            push(@orderedAttributes, $attribute);
         }
      }
   }
   #print("   sorted order = @orderedAttributes\n\n");

   if (@orderedAttributes) {
      $attributeDefinition = "<!ATTLIST $element";
      foreach $attribute (@orderedAttributes) {
         if ($isRequired{$attribute}) {
            $attributeDefinition .= "\n  $attribute CDATA #REQUIRED";
         } else {
            $attributeDefinition .= "\n  $attribute CDATA #OPTIONAL";
         }
      }
      $attributeDefinition .= ">\n";
      $result .= "$attributeDefinition\n";
   }
   $result .= "\n";

   return $result;
}


# sub numerically:
# For numeric sorting. Normal sorting is ascii. Usage 'sort numerically @list;'.
# It doesn't work if imported from another module, so it can't usefully be 
# exported either. For use in this module only.

sub numerically ()
{
   $a <=> $b;
}


return 1;
