package Generate::Element;

# Module:  Generate:Element:
# Purpose: Generates an ELEMENT DTD entry
# Author:  John Storrs, Laboratory for Micro Enterprise
# License: FSF GPL
# Version: $Id: Element.pm,v 1.2 2005/07/10 11:37:07 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   our @EXPORT_OK = qw(generateElement);
}

use LME::XmlParser;
use LME::XmlNode;
use Generate::Main qw(%hasElements);

# prototypes
sub generateElement($);


sub generateElement ($)
{
   my ($element) = @_;
   my ($result, $pattern, $matchPattern, $newPattern, $escapedPattern, 
       $definition, $tag, @elementContent, $i);

   # simplify element content patterns (pass 1)
   foreach $pattern (keys %hasElements) {

      # if pattern exactly matches 'tag+'
      if ($pattern =~ m/^([^ +]+)\+, $/) {
         $tag = $1;

         # any pattern exactly matching 'tag' is redundant
         delete $hasElements{$tag};

         # replace 'tag' in any pattern containing it with 'tag+'
         # but not if it already has a '+' suffix
         foreach $pattern (keys %hasElements) {
            $newPattern = $pattern;
            if ($newPattern =~ s/$tag, /$tag+, /g) {
               delete $hasElements{$pattern};
               $hasElements{$newPattern} = 1;
            }
         }
      }
   }

   # simplify element content patterns (pass 2)
   foreach $matchPattern (keys %hasElements) {
      $escapedPattern = $matchPattern;
      $escapedPattern =~ s/([+*?])/\\$1/g;  # protect '+', '*', '?'
      #print("MATCH PATTERN = '$matchPattern'\n");

      # if pattern exactly matches 'tag2, tag3+, ', and another pattern 
      # matches 'tag1, tag2, tag3+, tag4+, ', the first pattern is redundant
      # and tag1 and tag4 are optional
      foreach $pattern (keys %hasElements) {
         if ($pattern ne $matchPattern) {
            if ($pattern =~ m/$escapedPattern/) {
               $newPattern = $pattern;
               #print("PATTERN = $newPattern\n");
               $newPattern =~ s/$escapedPattern/XXX, /;  # hide match pattern
               $newPattern =~ s/([^+*?]), /$1?, /g;  # 'tag, ' -> 'tag?, '
               $newPattern =~ s/\+, /*, /g;  # 'tag+ ,' -> 'tag*, '
               $newPattern =~ s/XXX\?, /$matchPattern/;  # recover match
               #print("NEW PATTERN = $newPattern\n");
               delete $hasElements{$matchPattern};
               delete $hasElements{$pattern};
               $hasElements{$newPattern} = 1;
            }
         }
      }
   }

   # if there are several patterns and one is 'empty, ', the other
   # patterns are optional and the empty pattern is redundant
   $matchPattern = 'empty, ';
   if (grep(/$matchPattern/, keys(%hasElements))) {
      foreach $pattern (keys %hasElements) {
         if ($pattern ne $matchPattern) {
            $newPattern = $pattern;
            $newPattern =~ s/([^+*?]), /$1?, /g;  # 'tag, ' -> 'tag?, '
            $newPattern =~ s/\+, /*, /g;  # 'tag+ ,' -> 'tag*, '
            delete $hasElements{$matchPattern};
            delete $hasElements{$pattern};
            $hasElements{$newPattern} = 1;
         }
      }
   }

   # strip trailing ', ' from patterns
   @elementContent = ();
   foreach $pattern (keys(%hasElements)) {
      $pattern =~ s/, $//;
      push(@elementContent, $pattern);
   }

   # output element definition
   $definition = "<!ELEMENT $element ((";
   $definition .= join(')|(', @elementContent);
   $definition .= "))>";
   $result .= "$definition\n";

   # fix result for now
   $result =~ s/:text:/#PCDATA/g;
   $result =~ s/:cdata:/#PCDATA/g;

   return $result;
}


# sub numerically:
# For numeric sorting. Normal sorting is ascii. Usage 'sort numerically @list;'.
# It doesn't work if imported from another module, so it can't usefully be 
# exported either. For use in this module only.

sub numerically ()
{
   $a <=> $b;
}


return 1;
