package DocBook::Html::Graphics;

# Processes DocBook graphics elements: graphic, inlinegraphic.

# For graphics inclusions, note that when using pdflatex, eps files cannot be
# directly imported. The supported graphics files types are .jpg, .png, and
# .pdf. One way to convert eps to pdf is to use Sebastian Rahtz' epstopdf Perl
# script which is in /usr/bin. See the TeX FAQ under 'Imported graphics in
# PDFLateX'.

# $Id: Graphics.pm,v 1.2 2005/07/10 11:31:45 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{graphic} = \&graphic;
   $dispatchTable{inlinegraphic} = \&inlinegraphic;
};


sub graphic ($)
{
   my ($node) = @_;
   my ($result, $fileref, $width, $align);
   
   $fileref = $node->attribute('fileref');
   $width = $node->attribute('width');
   $align = $node->attribute('align');
   $align = "\L$align";

   if ($align eq "left") {
      $result .= "<img class=\"align-left\" src=\"$fileref\"";
   } else {
      if ($align eq "right") {
         $result .= "<img class=\"align-right\" src=\"$fileref\"";
      } else {
         $result = "<p><center>";
         $result .= "<img src=\"$fileref\"";
      }
   }
   if ($width) {
      if ($width !~ m/\d+%/) {
         print("graphic widths must be percentages\n");
      }
      $result .= " style=\"width: $width\">";
   } else {
      $result .= ">";
   }

   $result .= processChildren($node);  # should be empty

   $result .= "</img>";

   if (($align ne "left") && ($align ne "right")) {
      $result .= "</center></p>\n";  # end paragraph
   }

   return $result;
}


sub inlinegraphic ($)
{
   my ($node) = @_;
   my ($result, $fileref);
   
   $fileref = $node->attribute('fileref');

   $result .= "<img src=\"$fileref\">";

   $result .= processChildren($node);  # should be empty

   $result .= "</img>";

   return $result;
}


return 1;
