package DocBook::Html::Links;

# Processes DocBook link elements: anchor, link, ulink, xref.

# If an html anchor element has a title attribute, that will be used by modern 
# browsers (eg firefox) in a tooltip display when the cursor hovers over the 
# link.

# $Id: Links.pm,v 1.3 2005/09/10 12:05:31 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);
use DocBook::Html::Main qw(%pageById %xrefById);
use DocBook::Html::Sections qw($htmlDirectory);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{anchor} = \&anchor;
   $dispatchTable{link} = \&link;
   $dispatchTable{ulink} = \&ulink;
   $dispatchTable{xref} = \&xref;
};


sub anchor ($)
{
   my ($node) = @_;
   my ($result, $id);
   
   $id = $node->attribute('id');
   $result = "<a name=\"$id\"/>\n";

   $result .= processChildren($node);  # should be empty

   return $result;
}


sub link ($)
{
   my ($node) = @_;
   my ($result, $linkend, $pageId, $fileName);

   $linkend = $node->attribute('linkend');
   $pageId = $pageById{$linkend};
   if ($pageId) {
      #$fileName = "$htmlDirectory/${pageId}.html";
      $fileName = "${pageId}.html";
   } else {
      #$fileName = "$htmlDirectory/LINK_START${linkend}LINK_END.html";
      $fileName = "LINK_START${linkend}LINK_END.html";
   }
   $result = "<a href=\"$fileName#$linkend\">";

   $result .= processChildren($node);

   $result .= "</a>\n";

   return $result;
}


sub ulink ($)
{
   my ($node) = @_;
   my ($result, $url);
   
   $url = $node->attribute('url');
   if ($url =~ m/http/) {
      $result = "<a href=\"$url\" target=\"= new window\">";
   } else {
      $result = "<a href=\"$url\">";
   }

   $result .= processChildren($node);

   $result .= "</a>\n";

   return $result;
}


sub xref ($)
{
   my ($node) = @_;
   my ($result, $endterm, $linkend, $pageId, $xref, $fileName);

   $endterm = $node->attribute('endterm');
   $linkend = $node->attribute('linkend');
   $pageId = $pageById{$linkend};
   $xref = $xrefById{$linkend};
=hide
   if ($pageId) {
      #$fileName = "$htmlDirectory/${pageId}.html";
      $fileName = "${pageId}.html";
   } else {
      #$fileName = "$htmlDirectory/LINK_START${linkend}LINK_END.html";
      $fileName = "LINK_START${linkend}LINK_END.html";
   }
   $result = "<a href=\"$fileName#$linkend\">$endterm</a>\n";
=cut

   # WORK IN PROGRESS
   if ($xref) {
      $result .= $xref;
   } else {
      $result .= "XREF_START${linkend}XREF_END";
   }
   
   $result .= processChildren($node);  # should be empty

   return $result;
}


return 1;
