package DocBook::Html::Lists;

# Processes DocBook list elements: itemizedlist, orderedlist, segmentedlist,
# variablelist, listitem, term, varlistentry.

# $Id: Lists.pm,v 1.2 2005/07/10 11:31:45 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);
use DocBook::Html::Main qw(%xrefById);

our ($itemNumber);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{itemizedlist} = \&itemizedlist;
   $dispatchTable{orderedlist} = \&orderedlist;
   $dispatchTable{segmentedlist} = \&segmentedlist;
   $dispatchTable{variablelist} = \&variablelist;
   $dispatchTable{listitem} = \&listitem;
   $dispatchTable{term} = \&term;
   $dispatchTable{varlistentry} = \&varlistentry;
};


sub itemizedlist ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "<ul>\n";

   $result .= processChildren($node);

   $result .= "</ul>\n";

   return $result;
}


sub orderedlist ($)
{
   my ($node) = @_;
   my ($result);

   $itemNumber = 0;
   
   $result = "<ol>\n";

   $result .= processChildren($node);

   $result .= "</ol>\n";

   return $result;
}


sub segmentedlist ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "<dl>\n";

   $result .= processChildren($node);

   $result .= "</dl>\n";

   return $result;
}


sub variablelist ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "<dl>\n";

   $result .= processChildren($node);

   $result .= "</dl>\n";

   return $result;
}


sub listitem ($)
{
   my ($node) = @_;
   my ($result, $id, $parent);

   $id = $node->attribute('id');

   $itemNumber++;
   if ($id) {
      $xrefById{$id} = $itemNumber;
   }
   
   $parent = $tagStack[1];

   if ($parent eq 'varlistentry') {
      $result = "<dd>";
   } else {
      $result = "<li>";
   }

   $result .= processChildren($node);

   if ($parent eq 'varlistentry') {
      $result .= "</dd>\n";
   } else {
      $result .= "</li>\n";
   }

   return $result;
}


sub term ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "<dt><b>";

   $result .= processChildren($node);

   $result .= "</b></dt>";

   return $result;
}


sub varlistentry ($)
{
   my ($node) = @_;
   my ($result, $id);
   
   $result = "";

=hide
   # WORK IN PROGRESS - HOW TO AUTOMATE THIS?
   $id = $node->attribute('id');
   if ($id) {
      $result .= "<a id=\"$id\"/>";
   }
=cut

   $result .= processChildren($node);

   $result .= "";

   return $result;
}


return 1;
