package DocBook::Html::Titles;

# Processes DocBook title element.

# $Id: Titles.pm,v 1.3 2005/09/10 12:05:31 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);
use DocBook::Html::Sections
       qw($htmlDirectory $sectionIndex $sectionLevel $isSection $startingPage 
          $sectionNumbering $textNavigation $largestHeading $smallestHeading
          $menuItemSeparator @sectionIds @sectionTitle @sectionNumbers
          @sectionData @pageSectionLevel);
use DocBook::Html::Tables qw(%table);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{title} = \&title;
};


sub title ($)
{
   my ($node) = @_;
   my ($result, $i, $parent, $title, $headingSize, $id, $number, $pageId,
       $fileName, $sectionId, $previousIndex, $nextIndex, $pageLevel);

   $title = processChildren($node);

   $parent = $tagStack[1];
   if ($isSection) {
      $pageLevel = $pageSectionLevel[0];
      if ($largestHeading + $sectionLevel - $pageLevel < $smallestHeading) {
         $headingSize = $largestHeading + $sectionLevel - $pageLevel;
      } else {
         $headingSize = $smallestHeading;
      }
      $number = '';
      foreach $i (2 .. $sectionLevel) {
         $number .= "$sectionNumbers[$i].";
      }
      chop($number);
      if ($startingPage) {  # suppress title
         $result = '';
         $startingPage = 0;
      } else {  # emit title
         #$result = "<table border=\"0\">";
         #$result .= "<tr><td>";
         $result .= "<h$headingSize id=\"$sectionIds[0]\">";
         if ($sectionNumbering) {
            $result .= "$number ";
         }
         $result .= "$title";
         $result .= "</h$headingSize>\n";

         if ($textNavigation) {
            $pageId = $sectionData[$sectionIndex]{pageId};
            #$fileName = "$htmlDirectory/$pageId.html";
            $fileName = "$pageId.html";
            $result .= "<div class=\"section-map\">\n";
            $result .= "<a href=\"$fileName\">Top</a>";
            $previousIndex = $sectionIndex - 1;
            $sectionId = $sectionData[$previousIndex]{id};
            #$fileName = "$htmlDirectory/$pageId.html";
            $fileName = "$pageId.html";
            $result .= 
              "$menuItemSeparator<a href=\"$fileName#$sectionId\">Previous</a>";
            $nextIndex = $sectionIndex + 1;
            $sectionId = "s$nextIndex";
            #$fileName = "$htmlDirectory/$pageId.html";
            $fileName = "$pageId.html";
            $result .= 
               "$menuItemSeparator<a href=\"$fileName#$sectionId\">Next</a>";
            $result .= "</div>\n";
         }
      }
      unshift(@sectionTitle, $title);  # push onto stack
      $sectionData[$sectionIndex]{title} = $title;
      $sectionData[$sectionIndex]{number} = $number;
      $isSection = 0;
   } elsif ($parent eq 'table') {
      $table{name} = $title;
      $result = '';
   } else {
      $result = $title;
   }

   return $result;
}


return 1;
