package DocBook::Latex::Emphasis;

# Processes DocBook inline elements requiring emphasis:

# $Id: Emphasis.pm,v 1.2 2005/07/10 11:31:47 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw();
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   my ($key, $tag, @groupTags, %commandGroups);

   @{$commandGroups{bold}} = qw(application command hardware);

   @{$commandGroups{italic}} = qw(
guibutton guiicon guilabel guimenu guimenuitem guisubmenu);

   @{$commandGroups{mono}} = qw(
classname computeroutput constant email envar filename function interface
keycap keycode keycombo keysym literal mousebutton parameter prompt replaceable
returnvalue shortcut structfield structname symbol type userinput varargs
varname);

   # map tags to subroutines through dispatch table
   foreach $key (keys(%commandGroups)) {
      @groupTags = @{$commandGroups{$key}};
      foreach $tag (@groupTags) {
         $dispatchTable{$tag} = eval("\\\&$key");  # eg sect1 => \&section
      }
   }

   # build dispatch table
   $dispatchTable{emphasis} = \&emphasis;
};


sub bold ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\textbf{";

   $result .= processChildren($node);

   $result .= "}";

   return $result;
}


sub italic ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\textit{";

   $result .= processChildren($node);

   $result .= "}";

   return $result;
}


sub mono ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\texttt{";

   $result .= processChildren($node);

   $result .= "}";

   return $result;
}


# sub emphasis:
# This deals with <emphasis><emphasis> by depositing \textit{\textbf{content}},
# giving bold italic.

sub emphasis ($)
{
   my ($node) = @_;
   my ($result);
   
   if ($tagStack[1] eq 'emphasis') {
      $result = "\\textbf{";
   } else {
      $result = "\\textit{";
   }

   $result .= processChildren($node);

   $result .= "}";

   return $result;
}


return 1;
