package DocBook::Latex::Glossary;

# Processes DocBook glossary elements: glossary, glosslist, glossentry,
# glossterm, glossdef.

# $Id: Glossary.pm,v 1.2 2005/07/10 11:31:47 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{glossary} = \&glossary;
   $dispatchTable{glosslist} = \&glosslist;
   $dispatchTable{glossentry} = \&glossentry;
   $dispatchTable{glossterm} = \&glossterm;
   $dispatchTable{glossdef} = \&glossdef;
};


sub glossary ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = processChildren($node);

   return $result;
}


sub glosslist ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\begin{description}\n";

   $result .= processChildren($node);

   $result .= "\\end{description}\n";

   return $result;
}


sub glossentry ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\item ";

   $result .= processChildren($node);

   $result .= "\n";

   return $result;
}


# sub glossterm:
# glossentries can contain glossterm, acronym, abbrev, glossdef etc. We open 
# the html term below but do not close it so any acronym or abbrev is included
# in the term.

sub glossterm ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "[";

   $result .= processChildren($node);

   $result .= " ";

   return $result;
}


sub glossdef ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "] ";
   $result .= "\\item ";

   $result .= processChildren($node);

   $result .= "\n";

   return $result;
}


return 1;
