package DocBook::Latex::Graphics;

# Processes DocBook graphics elements: graphic, inlinegraphic.

# For graphics inclusions, note that when using pdflatex, eps files cannot be
# directly imported. The supported graphics files types are .jpg, .png, and
# .pdf. One way to convert eps to pdf is to use Sebastian Rahtz' epstopdf Perl
# script which is in /usr/bin. See the TeX FAQ under 'Imported graphics in
# PDFLateX'. A recent version of ImageMagick is required for graphics scaling 
# to work properly.

# $Id: Graphics.pm,v 1.2 2005/07/10 11:31:47 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{graphic} = \&graphic;
   $dispatchTable{inlinegraphic} = \&inlinegraphic;
};


sub graphic ($)
{
   my ($node) = @_;
   my ($result, $fileref, $basename, $extension, $scale, $density);

   $fileref = $node->attribute('fileref');
   $scale = $node->attribute('scale');
   $fileref =~ m/\.(\w+)$/;
   $extension = $1;
   if ($extension) {
      $basename = $fileref;
      $basename =~ s/\.$extension$//;
   } else {
      print("ERROR: graphic file must have an extension\n");
      exit(-1);
   }

   # convert to pdf format with scaling if required
   if ($scale) {
      $density = 72/$scale;
      system
         ("convert -density $density ${basename}.$extension ${basename}.pdf");
   } else {
      if ($extension ne 'pdf') {
         system("convert ${basename}.$extension ${basename}.pdf");
      }
   }

   $result = processChildren($node);  # should be empty
   if ($result) {
      print("ERROR: element graphic should have no children\n");
      exit(-1);
   }

   $result = "\n";  # force paragraph
   $result .= "\\begin{center}\n";
   $result .= "\\includegraphics{${basename}.pdf}\n";
   $result .= "\\end{center}\n";
   $result .= "\n";  # end paragraph

   return $result;
}


sub inlinegraphic ($)
{
   my ($node) = @_;
   my ($result, $fileref, $basename);

   $fileref = $node->attribute('fileref');
   if ($fileref =~ m/\.gif$/) {
      $basename = $fileref;
      $basename =~ s/\.gif//;
      system("convert ${basename}.gif ${basename}.png");
   }
   
   $result = processChildren($node);  # should be empty
   if ($result) {
      print("ERROR: element graphic should have no children\n");
   }

   $result = "\n";  # force paragraph
   $result .= "\\includegraphics{${basename}.png}";
   $result .= "\n";  # end paragraph

   return $result;
}


return 1;
