package DocBook::Latex::Links;

# Processes DocBook link elements: anchor, link, ulink, xref.

# $Id: Links.pm,v 1.2 2005/07/10 11:31:47 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{anchor} = \&anchor;
   $dispatchTable{link} = \&link;
   $dispatchTable{ulink} = \&ulink;
   $dispatchTable{xref} = \&xref;
};


sub anchor ($)
{
   my ($node) = @_;
   my ($result, $id);

   $id = $node->attribute('id');

   $result = processChildren($node);  # should be empty

   $result .= "\\label{$id}";

   return $result;
}


sub link ($)
{
   my ($node) = @_;
   my ($result, $linkend);

   $linkend = $node->attribute('linkend');

   $result = processChildren($node);

   # THIS PRINTS ?? SO COMMENTED OUT FOR NOW
   #$result .= " (\\ref{$linkend})";

   return $result;
}


sub ulink ($)
{
   my ($node) = @_;
   my ($result, $url, $fixedUrl);
   
   $url = $node->attribute('url');
   $fixedUrl = $url;
   $fixedUrl =~ s|\.|\.\\-|g;  # insert optional hyphenation point after '.'
   $fixedUrl =~ s|/|/\\-|g;  # insert optional hyphenation point after '/'
   $fixedUrl =~ s|_|\\_|g;  # escape '_' in url 

   $result = processChildren($node);

   if ($url =~ m/http/) {
      $result = "\\href{$url}{$result} ($fixedUrl)";
      return $result;
   }

   if ($url =~ m/\/pdf\//) {  # strip unwanted pdf links
      return '';
   }

   return $result;
}


sub xref ($)
{
   my ($node) = @_;
   my ($result, $endterm, $linkend);

   $endterm = $node->attribute('endterm');
   $linkend = $node->attribute('linkend');

   $result = "$endterm (\\ref{$linkend})\n";
   # Have to deal with cases where the reference doesn't resolve -
   # eg when part of a document is printed as an article.

   $result .= processChildren($node);  # should be empty

   return $result;
}


return 1;
