package DocBook::Latex::Lists;

# Processes DocBook list elements: itemizedlist, orderedlist, segmentedlist,
# variablelist, listitem, term, varlistentry.

# $Id: Lists.pm,v 1.2 2005/07/10 11:31:47 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{itemizedlist} = \&itemizedlist;
   $dispatchTable{orderedlist} = \&orderedlist;
   $dispatchTable{segmentedlist} = \&segmentedlist;
   $dispatchTable{variablelist} = \&variablelist;
   $dispatchTable{listitem} = \&listitem;
   $dispatchTable{term} = \&term;
   $dispatchTable{varlistentry} = \&varlistentry;
};


sub itemizedlist ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\begin{itemize}\n";

   $result .= processChildren($node);

   $result .= "\\end{itemize}\n";

   return $result;
}


sub orderedlist ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\begin{enumerate}\n";

   $result .= processChildren($node);

   $result .= "\\end{enumerate}\n";

   return $result;
}


sub segmentedlist ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\begin{description}\n";

   $result .= processChildren($node);

   $result .= "\\end{description}\n";

   return $result;
}


sub variablelist ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\begin{description}\n";

   $result .= processChildren($node);

   $result .= "\\end{description}\n";

   return $result;
}


sub listitem ($)
{
   my ($node) = @_;
   my ($result, $parent);
   
   $parent = $tagStack[1];

   if ($parent ne 'varlistentry') {
      $result = "\\item ";
   }

   $result .= processChildren($node);

   $result .= "\n";

   return $result;
}


sub term ($)
{
   my ($node) = @_;
   my ($result, $term);
   
   $result = "[";

   $term = processChildren($node);
   $term =~ s/^\s+//;  # strip enclosing space
   $term =~ s/\s+$//;
   $result .= $term;

   $result .= "] ";

   return $result;
}


sub varlistentry ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "\\item ";

   $result .= processChildren($node);

   $result .= "\n";

   return $result;
}


return 1;
