package DocBook::Latex::Text;

# Processes DocBook text elements: cdata, text.

# $Id: Text.pm,v 1.2 2005/07/10 11:31:47 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{':cdata:'} = \&cdata;
   $dispatchTable{':text:'} = \&text;
};


sub cdata ($)
{
   my ($node) = @_;
   my ($cdata);

   $cdata = $node->cdata();
   $cdata =~ s/^\s+//;  # clip enclosing white space
   $cdata =~ s/\s+$//;

   return $cdata;
}


sub text ($)
{
   my ($node) = @_;
   my ($text, $parent);

   $text = $node->text();
   $parent = $tagStack[1];

   if ($parent ne 'math') {
      # expand some xml character entities
      $text =~ s/\&amp;/\&/g;
      $text =~ s/\&lt;/\</g;
      $text =~ s/\&gt;/\>/g;

      if (($parent ne 'literallayout') && 
          ($parent ne 'programlisting') && 
          ($parent ne 'screen') && 
          ($parent ne 'system')) {
         # allow backslashes in input text
         # and see below!
         $text =~ s/\\/XXbackslashXX/g;

         # fix tex command characters (Kopka p21)
         $text =~ s/\$/\\\$/g;

         $text =~ s/\&lt;/\$\<\$/g;
         $text =~ s/\&gt;/\$\>\$/g;

         $text =~ s/\&/\\\&/g;
         $text =~ s/\%/\\\%/g;
         $text =~ s/\#/\\\#/g;
         $text =~ s/_/\\_/g;
         $text =~ s/\{/\\\{/g;
         $text =~ s/\}/\\\}/g;

         # special treatment for ~ and ^ accent chars
         $text =~ s/\^/\\verb#^#/g;
         $text =~ s/\~/\\verb#~#/g;
         $text =~ s/\|/\\verb#|#/g;

         # complete backslash handling!
         $text =~ s/XXbackslashXX/\\verb#\\#/g;
      }

   } else {  # parent is latex

      # strip any surrounding white space 
      $text =~ s/^\s*//;
      $text =~ s/\s*$//;
   }
   $text =~ s/\s+$/ /;  # collapse terminal white space
   $text .= "\n";  # append newline

   # replace character entities - limited for now
   $text =~ s/&Omega;/\\\\Omega /g;

   # strip leading newlines for verbatim
   $text =~ s/^\n+//;  

   return $text;
}


return 1;
