package DocBook::Rtf::Links;

# Processes DocBook link elements: anchor, link, ulink, xref.

# $Id: Links.pm,v 1.1 2005/07/10 11:26:37 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{anchor} = \&anchor;
   $dispatchTable{link} = \&link;
   $dispatchTable{ulink} = \&ulink;
   $dispatchTable{xref} = \&xref;
};


sub anchor ($)
{
   my ($node) = @_;
   my ($result, $id);

   $id = $node->attribute('id');

   $result = processChildren($node);  # should be empty

   #$result .= "\\label{$id}";

   return $result;
}


sub link ($)
{
   my ($node) = @_;
   my ($result, $linkend);

   $linkend = $node->attribute('linkend');

   $result = processChildren($node);

   #$result .= " (\\ref{$linkend})";

   return $result;
}


sub ulink ($)
{
   my ($node) = @_;
   my ($result, $url, $fixedUrl);
   
   $url = $node->attribute('url');

   $result = "{\\field{\\\*\\fldinst{HYPERLINK \"$url\"}}{\\fldrslt{\\ul\n";

   $result .= processChildren($node);

   $result .= "}}}\n";

   return $result;
}


sub xref ($)
{
   my ($node) = @_;
   my ($result, $endterm, $linkend);

   $endterm = $node->attribute('endterm');
   $linkend = $node->attribute('linkend');

   $result = "$endterm (\\ref{$linkend})\n";
   # Have to deal with cases where the reference doesn't resolve -
   # eg when part of a document is printed as an article.

   $result .= processChildren($node);  # should be empty

   return $result;
}


return 1;
