package DocBook::Rtf::Main;

# $Id: Main.pm,v 1.1 2005/07/10 11:26:35 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   our @EXPORT_OK = qw($preamble $postfix);
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack $result);
use DocBook::Rtf::Emphasis;
use DocBook::Rtf::Footnotes;
use DocBook::Rtf::Graphics;
use DocBook::Rtf::Glossary;
use DocBook::Rtf::Inline;
use DocBook::Rtf::Links;
use DocBook::Rtf::Lists;
use DocBook::Rtf::Qandaset;
use DocBook::Rtf::Structure;
use DocBook::Rtf::Tables;
use DocBook::Rtf::Text;
use DocBook::Rtf::Titles;
use DocBook::Rtf::Verbatim;

our ($preamble, $postfix);

sub initialize ()
{
   my ($key, $tag, @groupTags, $routine, $symname, $sym, @sym, %sym, 
       %commandGroups);

   @{$commandGroups{noaction}} = qw(
accel ackno action address answer area areaset areaspec arg
artpagenums attribution audiodata audioobject beginpage bibliodiv biblioentry
bibliography bibliomisc bibliomixed bibliomset biblioset callout calloutlist
caption caution citation citerefentry citetitle city cmdsynopsis co collab
collabname colophon confdates confgroup confnum confsponsor conftitle
contractnum contractsponsor contrib copyright corpauthor corpname country
database date dedication edition editor entrytbl epigraph errorcode errorname
errortype example fax figure firstname firstterm foreignphrase funcdef
funcparams funcprototype funcsynopsis funcsynopsisinfo graphicco group
highlights holder honorific imagedata imageobject imageobjectco important
invpartnumber isbn issn issuenum itermset jobtitle keyword keywordset label
legalnotice lineage lineannotation lot lotentry manvolnum markup medialabel
mediaobject mediaobjectco member menuchoice modespec msg msgaud msgentry
msgexplan msginfo msglevel msgmain msgorig msgrel msgset msgsub msgtext note
objectinfo option optional orgdiv orgname otheraddr othercredit othername
pagenums paramdef partintro phone phrase pob postcode primary primaryie
printhistory procedure productname productnumber programlistingco property
pubsnumber question quote refclass refdescriptor refentry refentrytitle
reference refmeta refmiscinfo refname refnamediv refpurpose refsect1
refsect1info refsect2 refsect2info refsect3 refsect3info refsynopsisdiv
refsynopsisdivinfo releaseinfo revhistory revision revnumber revremark sbr
screenco screeninfo screenshot secondary secondaryie sect1info sect2info
sect3info sect4info sect5info sectioninfo see seealso seealsoie seeie seg
seglistitem segtitle seriesvolnums sgmltag shortaffil sidebar state step street
subject subjectset subjectterm subscript substeps subtitle superscript surname
synopfragment synopfragmentref synopsis systemitem tertiary tertiaryie
textobject tip toc tocback tocchap tocentry tocfront toclevel1 toclevel2
toclevel3 toclevel4 toclevel5 tocpart token trademark videodata videoobject
void volumenum warning wordasword year);

   @{$commandGroups{special}} = qw(
abstract affiliation alt anchor appendix article author authorblurb authorgroup
authorinitials blockquote book bookinfo bridgehead colspec emphasis entry
equation footnote footnoteref formalpara glossary glossdef glossdiv glossentry
glosslist glosssee glossseealso glossterm graphic index indexdiv indexentry
indexterm informalequation informalexample informalfigure inlineequation
inlinegraphic inlinemediaobject itemizedlist link listitem literallayout
olink orderedlist para part preface programlisting pubdate publisher
publishername qandadiv qandaentry qandaset row screen segmentedlist set
setindex setinfo simpara simplelist spanspec tbody term tfoot tgroup thead
title titleabbrev ulink variablelist varlistentry xref :cdata: :text:);

   # map tags to subroutines through dispatch table
   foreach $key (keys(%commandGroups)) {
      @groupTags = @{$commandGroups{$key}};
      if ($key eq 'special') {
         foreach $tag (@groupTags) {
            $routine = $tag;
            $routine =~ s/://g;  # strip ':' in :text: and :cdata:

            # check if routine is defined in package symbol table
            # see Programming Perl, O'Reilly, p293
            if (($DocBook::Rtf::{$routine}) &&  # is routine symbol listed?
                (defined(&{$DocBook::Rtf::{$routine}}))) {  # and a routine?
                  $dispatchTable{$tag} = eval("\\\&$routine"); 
                              # eg book => \&book
            } else {
               # print("no routine defined for tag $tag\n");
            }
         }
      } else {
         foreach $tag (@groupTags) {
            $dispatchTable{$tag} = eval("\\\&$key");  # eg sect1 => \&section
         }
      }
   }

   # rtf preamble
   $preamble = <<EOT;
{\\rtf1\\ansi\\deff0{\\fonttbl
{\\f0 Palatino Linotype;}
{\\f1 Courier New;}}
\\deflang1033\\ftnbj\\ftnrestart\\widowctrl\\hyphauto\\pgnstart1
\\margl1134\\margr1134\\margt1440\\margb1440
{\\colortbl;\\red255\\green0\\blue0;\\red0\\green0\\blue255;}
{\\header\\pard\\qr\\plain\\f0\\chpgn\\par}
EOT

   DocBook::Rtf::Emphasis::initialize();
   DocBook::Rtf::Footnotes::initialize();
   DocBook::Rtf::Graphics::initialize();
   DocBook::Rtf::Glossary::initialize();
   DocBook::Rtf::Inline::initialize();
   DocBook::Rtf::Links::initialize();
   DocBook::Rtf::Lists::initialize();
   DocBook::Rtf::Qandaset::initialize();
   DocBook::Rtf::Structure::initialize();
   DocBook::Rtf::Tables::initialize();
   DocBook::Rtf::Text::initialize();
   DocBook::Rtf::Titles::initialize();
   DocBook::Rtf::Verbatim::initialize();

   $preamble .= <<EOT;
EOT

   $postfix = '}';
};


sub finalize ()
{
}


sub noaction ($)
{
   my ($node) = @_;
   my ($result);

   $result = processChildren($node);

   return $result;
}


return 1;
