package DocBook::Rtf::Structure;

# Processes DocBook elements which give documents their primary structure:
# part, book, article, section, sect1, sect2, sect3, sect4, sect5, para.

# Rtf treats the higher level sectioning commands differently to paragraph
# and subparagraph (defined as 'run-in types'). See Kopka & Daly p348.  For
# normal DocBook processing we need to change this so all sectioning commands
# behave the same. Hence renewcommand...startsection. This also fixes a bug in
# Rtf, where a longtable as the only element in a paragraph element comes 
# before the paragraph heading! This is nothing to do with floats, because 
# longtables don't float.

# $Id: Structure.pm,v 1.1 2005/07/10 11:26:34 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw();
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);
use DocBook::Rtf::Main qw($preamble $postfix);

# provate global variables
our ($sectionDepth, @sectionCommands);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{part} = \&part;
   $dispatchTable{book} = \&book;
   $dispatchTable{article} = \&article;
   $dispatchTable{chapter} = \&section;
   $dispatchTable{section} = \&section;
   $dispatchTable{sect1} = \&section;
   $dispatchTable{sect2} = \&section;
   $dispatchTable{sect3} = \&section;
   $dispatchTable{sect4} = \&section;
   $dispatchTable{sect5} = \&section;
   $dispatchTable{simplesect} = \&section;
   $dispatchTable{para} = \&para;

   # define Rtf section command strings and initial depth
   @sectionCommands = 
      qw(chapter section subsection subsubsection paragraph subparagraph
         subparagraph subparagraph subparagraph subparagraph subparagraph);
   $sectionDepth = 0;

   # add to rtf preamble if required
   $preamble .= <<EOT;
EOT

};


sub book ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = $preamble;

   $result .= processChildren($node);

   $result .= $postfix;

   return $result;
}


sub article ($)
{
   my ($node) = @_;
   my ($result);
   
   $sectionDepth++;

   # fix preamble
   #$preamble =~ s/{book}/{article}/;

   $result = $preamble;

   $result .= processChildren($node);

   $result .= $postfix;

   $sectionDepth--;

   return $result;
}


sub section ($)
{
   my ($node) = @_;
   my ($result, $id);
   
   $id = $node->attribute('id');

   #$result = "\\$sectionCommands[$sectionDepth]";
   $sectionDepth++;

   $result .= processChildren($node);

   #$result .= "\n\\label{$id}\n";

   $sectionDepth--;

   return $result;
}


sub para ($)
{
   my ($node) = @_;
   my ($result);
   
   $result = "{\\pard\\fs22\\sa300\n";

   $result .= processChildren($node);

   $result .= "\\par}\n";

   return $result;
}


return 1;
