package DocBook::Rtf::Text;

# Processes DocBook text elements: cdata, text.

# $Id: Text.pm,v 1.1 2005/07/10 11:26:40 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{':cdata:'} = \&cdata;
   $dispatchTable{':text:'} = \&text;
};


sub cdata ($)
{
   my ($node) = @_;
   my ($cdata);

   $cdata = $node->cdata();
   $cdata =~ s/^\s+//;  # clip enclosing white space
   $cdata =~ s/\s+$//;

   return $cdata;
}


sub text ($)
{
   my ($node) = @_;
   my ($text, $parent);

   $text = $node->text();
   $parent = $tagStack[1];

   if ($parent ne 'latex') {
      # expand some xml character entities
      $text =~ s/\&amp;/\&/g;
      $text =~ s/\&lt;/\</g;
      $text =~ s/\&gt;/\>/g;

      if (($parent ne 'literallayout') && 
          ($parent ne 'programlisting') && 
          ($parent ne 'screen')) {
         # convert white space to single space
         $text =~ s/\s+/ /g;
      }

      # allow special rtf chars in input text
      $text =~ s/\\/\\\\/g;
      $text =~ s/\{/\\{/g;
      $text =~ s/\}/\\}/g;

   } else {  # parent is latex

      # WITH RTF, WE HAVE PROBLEMS HERE

      # strip any surrounding white space 
      $text =~ s/^\s*//;
      $text =~ s/\s*$//;
   }
   $text =~ s/\s+$/ /;  # collapse terminal white space

   # replace character entities - limited for now
   $text =~ s/&Omega;/\\\\Omega /g;

   return $text;
}


return 1;
