package Extensions::Html::Division;

# Processes division elements: div, span. These propagate verbatim through to
# the generated html, allowing full control of document style through css
# stylesheets.

# $Id$


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{div} = \&div;
   $dispatchTable{span} = \&span;
};


sub div ($)
{
   my ($node) = @_;
   my ($result, $class, $id);

   $class = $node->attribute('class');
   $id = $node->attribute('id');

   if (!defined($class) && !defined($id)) {
      print("useless div without class or id found!");
   }
   
   $result = "<div";
   if (defined($class)) {
      $result .= " class=\"$class\"";
   }
   if (defined($id)) {
      $result .= " id=\"$id\"";
   }
   $result .= ">\n";

   $result .= processChildren($node);

   $result .= "</div>\n";

   return $result;
}


sub span ($)
{
   my ($node) = @_;
   my ($result, $class, $id);

   $class = $node->attribute('class');
   $id = $node->attribute('id');

   if (!defined($class) && !defined($id)) {
      print("useless span without class or id found!");
   }
   
   $result = "<span";
   if (defined($class)) {
      $result .= " class=\"$class\"";
   }
   if (defined($id)) {
      $result .= " id=\"$id\"";
   }
   $result .= ">\n";

   $result .= processChildren($node);

   $result .= "</span>\n";

   return $result;
}


return 1;
