package Extensions::Html::Exec;

# Processes execution extension elements: perl, system. These elements
# contain scripts which are executed, and the results inserted inline.

# $Id $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{'perl'} = \&perl;
   $dispatchTable{system} = \&system;
};


sub perl($)
{
   my ($node) = @_;
   my ($program, $result);

   $program = <<EOT;
#! /usr/bin/perl -wT

use strict;

EOT

   $program .= processChildren($node);

   # expand special entities
   $program =~ s/\&lt;/</g;
   $program =~ s/\&gt;/>/g;

   # execute program and return result
   $result = `$program`;

   return $result;
}


sub system ($)
{
   my ($node) = @_;
   my ($cwd, $showCommand, $hideAll, $maxLines, $prefix, $program, $result,
       $output, $command, $i, $line, @commands, @lines);

   $cwd = $node->attribute('cwd');
   $showCommand = $node->attribute('showCommand');
   $hideAll = $node->attribute('hideAll');
   $maxLines = $node->attribute('maxLines');
   if (!$cwd) {
      $cwd = '.';
   }
   if (!$showCommand) {
      $showCommand = 'yes';
   }
   if (!$hideAll) {
      $hideAll = 'no';
   }
   if (!$maxLines) {
      $maxLines = 10;
   }

   $program .= processChildren($node);
   $program =~ s/\&lt;/</g;
   $program =~ s/\&gt;/>/g;
   $program =~ s/^\s+//;
   $program =~ s/\s+$//;
   @commands = split(/\n+/, $program);

   $result = '';
   $prefix= '#! /bin/bash';
   foreach $command (@commands) {
      if ($showCommand eq 'yes') {
         $result .= '$ ' . "$command\n";
      }

      # execute program
      $program = "$prefix\n cd $cwd; $command";

      $output = `$program 2>&1`;
      @lines = split(/\n/, $output);
      if (scalar(@lines) > $maxLines) {
         $output = '';
         foreach $i (1..$maxLines) {
            $output .= "$lines[$i]\n";
         }
         $output .= "......\n";
      }

      $result .= $output;
   }

   $result =~ s/\n+/\n/g;

   if ($hideAll eq 'yes') {
      $result = '';
   }

   return $result;
}


return 1;
