package Extensions::Html::Graphics;

# Processes graphical extension elements: graphviz, gnuplot. These elements
# contain programs which are executed and the results displayed as graphics.

# $Id $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);

our ($imageId);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{graphviz} = \&graphviz;
   $dispatchTable{gnuplot} = \&gnuplot;

   $imageId = 0;
};


sub graphviz ($)
{
   my ($node) = @_;
   my ($program, $result, $fileName);

   $program = processChildren($node);

   # expand special entities
   $program =~ s/\&lt;/</g;
   $program =~ s/\&gt;/>/g;

   # process program and generate image file
   $fileName = "gv$imageId";
   open(FILE, ">$fileName.dot") or die("can't open $fileName.dot\n");
   print(FILE $program);
   close(FILE) or die("can't close $fileName.dot\n");
   system("dot -Tps $fileName.dot >$fileName.ps");
   system("convert $fileName.ps $fileName.png");
   system("mv $fileName.png ../graphics");
   $imageId++;

   $result = "<center><img src=\"../graphics/$fileName.png\"></center>\n";

   return $result;
}


sub gnuplot ($)
{
   my ($node) = @_;
   my ($program, $result, $fileName);

   $program = processChildren($node);

   # expand special entities
   $program =~ s/\&lt;/</g;
   $program =~ s/\&gt;/>/g;

   # process program and generate image file
   $fileName = "gp$imageId";
   open(FILE, ">$fileName.plot") or die("can't open $fileName.plot\n");
   print(FILE $program);
   close(FILE) or die("can't close $fileName.plot\n");
   system("gnuplot $fileName.plot");
   system("convert $fileName.ps $fileName.png");
   system("mv $fileName.png ../graphics");
   $imageId++;

   $result = "<center><img src=\"graphics/$fileName.png\"></center>\n";

   return $result;
}


return 1;
