package Extensions::Latex::Graphics;

# Processes graphical extension elements: graphviz, gnuplot. These elements
# contain programs which are executed and the results displayed as graphics.

# $Id: Graphics.pm,v 1.2 2005/07/10 11:32:07 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   # our @EXPORT_OK = qw()
}

use LME::XmlNode;
use Publish qw(processChildren %dispatchTable @tagStack);

our ($imageId);


sub initialize ()
{
   # build dispatch table
   $dispatchTable{graphviz} = \&graphviz;
   $dispatchTable{gnuplot} = \&gnuplot;

   $imageId = 0;
};


sub graphviz ($)
{
   my ($node) = @_;
   my ($program, $result, $fileName);

   $program = processChildren($node);

   # expand special entities
   $program =~ s/\&lt;/</g;
   $program =~ s/\&gt;/>/g;

   # fix Latex fix!
   $program =~ s/\\\$/\$/g;
   $program =~ s/\\\&/&/g;
   $program =~ s/\\\%/%/g;
   $program =~ s/\\\#/#/g;
   $program =~ s/\\\_/_/g;
   $program =~ s/\\\{/{/g;
   $program =~ s/\\\}/}/g;
   $program =~ s/\\\~/~/g;
   $program =~ s/\\\^/^/g;

   # process program and generate image file
   $fileName = "gv$imageId";
   open(FILE, ">$fileName.dot") or die("can't open $fileName.dot\n");
   print(FILE $program);
   close(FILE) or die("can't close $fileName.dot\n");
   system("dot -Tps $fileName.dot >$fileName.ps");
   #system("ps2pdf $fileName.ps");
   system("epstopdf $fileName.ps");
   $imageId++;

   $result = "\n";  # force paragraph
   $result .= "\\begin{center}\n";
   $result .= "\\includegraphics{$fileName.pdf}";
   $result .= "\\end{center}\n";
   $result .= "\n";  # end paragraph

   return $result;
}


sub gnuplot ($)
{
   my ($node) = @_;
   my ($program, $result, $fileName);

   $program = processChildren($node);

   # expand special entities
   $program =~ s/\&lt;/</g;

   # fix Latex fix!
   $program =~ s/\\{/{/g;
   $program =~ s/\\}/}/g;
   $program =~ s/\\#/#/g;

   # process program and generate image file
   $fileName = "gp$imageId";
   open(FILE, ">$fileName.plot") or die("can't open $fileName.plot\n");
   print(FILE $program);
   close(FILE) or die("can't close $fileName.plot\n");
   system("gnuplot $fileName.plot");
   system("ps2pdf plot.ps $fileName.pdf");
   $imageId++;

   $result = "\n";  # force paragraph
   $result .= "\\begin{center}\n";
   $result .= "\\includegraphics{$fileName.pdf}";
   $result .= "\\end{center}\n";
   $result .= "\n";  # end paragraph

   return $result;
}


return 1;
