package Publish;

# Module:  Publish
# Purpose: Publish XML driver module.
# Author:  John Storrs, Laboratory for Micro Enterprise
# License: FSF GPL
# Version: $Id: Publish.pm,v 1.2 2005/07/10 11:32:02 lme Exp $


use strict;

BEGIN {
   require Exporter;
   our @ISA = qw(Exporter);
   our @EXPORT_OK = 
          qw(publish processChildren @tagStack %dispatchTable $result);
}

use LME::XmlParser;
use LME::XmlNode;

# you must add new main modules here
use DocBook::Html::Main;
use DocBook::Html::Sections qw($currentPageId);
use DocBook::Latex::Main;
use DocBook::Rtf::Main;
use Extensions::Html::Main;
use Extensions::Latex::Main;


# forward declarations
sub publish($$$);
sub process($);
sub processChildren($);

# global variables
our ($result, $format, @tagStack, %dispatchTable);


# sub publish:
# The arguments to this subroutine are
#    (1) document markup (eg DocBook, Extensions);
#    (2) output format (eg Latex, Html).  FOR NOW, LIMITED TO 1 OF THESE AT A 
#                                         TIME
# As a result of the eval statements below, Main sets up the dispatch table 
# for this argument pair, so that the required element processing functions
# are called as the document node tree is traversed.

sub publish ($$$)
{
   my ($configuration, $document, $shortName) = @_;
   my ($xml, $config, $markup, $root, $fileName, @markups, @formats,
       $latexFormat, @latexFormats, $i, @variables, $variable, $variableName, 
       $variableValue, $status, @modules, $module, $moduleName);

   # import configuration file
   $config = parseXml(importFile($configuration));
   @markups = split(/,?\s+/, $config->attribute('markups'));
   @formats = split(/,?\s+/, $config->attribute('formats'));
   @latexFormats = ();
   if (grep(/Pdf/, @formats)) {
      if (!grep(/Latex/, @formats)) {
         push(@formats, 'Latex');
      }
      @formats = grep(!/Pdf/, @formats);
      push(@latexFormats, 'Pdf');
   }
   @modules = $config->nodeList('module');
   foreach $module (@modules) {
      $moduleName = $module->attribute('name');
      eval("use $moduleName");
      @variables = $module->nodeList('variable');
      foreach $variable (@variables) {
         $variableName = $variable->attribute('name');
         $variableValue = $variable->attribute('value');
         $status = eval("\$$variableName = '$variableValue';");
         if (!defined($status)) {
            print("variable $moduleName\:\:$variableName not recognised\n");
         }
      }
   }
   $config->destroy();

   # build document tree
   startParse(importFile($document));
   $root = nextNode();

   foreach $format (@formats) {
      print("generating $format\n");

      # initialize system
      @tagStack = ();
      %dispatchTable = ();
      $result = '';
      foreach $markup (@markups) {
         eval("$markup\:\:$format\:\:Main\:\:initialize();");
              # eg 'DocBook::Latex::Main::initialize();'
      }
   
      # process document tree
      process($root);
   
      # finalize system
      # NOW THAT FILE GENERATION IS HANDLED DOWNSTREAM, 
      # THIS SHOULDN'T BE REQUIRED.
      foreach $markup (@markups) {
         eval("$markup\:\:$format\:\:Main\:\:finalize();");
              # eg 'DocBook::Latex::Main::finalize();'
      }
   
      if ($format eq 'Latex') {
         $fileName = 'result.tex';
         open(FILE, ">$fileName") or die("can't open $fileName\n");
         print(FILE $result);
         close(FILE) or die("can't close $fileName\n");
         foreach $latexFormat (@latexFormats) {
            if ($latexFormat eq 'Pdf') {
               print("generating Pdf\n");
               system("pdflatex result > log");
               system("pdflatex result >> log");
               system("pdflatex result >> log");
               system("mv result.pdf ../pdf/$shortName.pdf");
            }
         }
      }
   
      if ($format eq 'Rtf') {
         $fileName = "../rtf/$shortName.rtf";
         open(FILE, ">$fileName") or die("can't open $fileName\n");
         print(FILE $result);
         close(FILE) or die("can't close $fileName\n");
      }
   }
}


sub process ($)
{
   my ($node) = @_;
   my ($tag);

   # get current tag and add to list
   $tag = $node->tag();
   unshift(@tagStack, $tag);

   if (defined($dispatchTable{$tag})) {
      $result = &{$dispatchTable{$tag}}($node);
   } else {
      $result = default($node);
   }
}


sub processChildren ($)
{
   my ($node) = @_;
   my ($result, $childResult, @children, $child, $tag, $parentTag, $id);

   $result = '';

   $parentTag = $node->tag();

   while ($child = nextChild($node)) {
      # get child tag and push on tag stack
      $tag = $child->tag();
      unshift(@tagStack, $tag);

      #print("processing child '$tag' of '$parentTag'\n");

      if ($format eq "Html") {
         # record html page id for this child id, for linking
         $id = $child->attribute('id');
         if ($id) {
            $pageById{$id} = $currentPageId;
         }
      }
   
      # call routine for child
      if (defined($dispatchTable{$tag})) {
         $childResult = &{$dispatchTable{$tag}}($child);
      } else {
         $childResult = default($child);
      }

      if (defined($childResult)) {
         $result .= $childResult;
      } else {
         $result .= '';
         print("ERROR: undefined result for tag $tag\n");
      } 
      
      # pop child tag from tag stack
      shift(@tagStack);
   }

   return $result;
}


sub default ($)
{
   my ($node) = @_;
   my ($unwanted, $result);

   print("no routine defined for $tagStack[0]\n");
   $unwanted = processChildren($node);
   $result = "processxml: nothing generated for $tagStack[0]\n";

   return $result;
}


return 1;
